@@ -18,13 +18,12 @@ import base64
 import binascii
 from decimal import ROUND_HALF_UP, Decimal, localcontext
 from distutils.util import strtobool
-import struct
 from typing import TYPE_CHECKING, Any, Dict, Optional
 
 from aiohomekit.exceptions import CharacteristicPermissionError, FormatError
-from aiohomekit.model.mixin import ToDictMixin
 from aiohomekit.protocol.statuscodes import HapStatusCodes
 from aiohomekit.protocol.tlv import TLV, TlvParseException
+from aiohomekit.tlv8 import tlv_array
 
 from .characteristic_formats import CharacteristicFormats
 from .characteristic_types import CharacteristicsTypes
@@ -59,7 +58,7 @@ INTEGER_TYPES = [
 NUMBER_TYPES = INTEGER_TYPES + [CharacteristicFormats.float]
 
 
-class Characteristic(ToDictMixin):
+class Characteristic:
     def __init__(self, service: "Service", characteristic_type: str, **kwargs) -> None:
         self.service = service
         self.iid = service.accessory.get_next_id()
@@ -84,33 +83,36 @@ class Characteristic(ToDictMixin):
         self.valid_values = self._get_configuration(kwargs, "valid_values", None)
         self.valid_values_range = None
 
-        self.value = None
+        self._value = None
 
         if CharacteristicPermissions.paired_read not in self.perms:
             return
 
         if "value" in kwargs:
-            self.value = kwargs["value"]
+            self._value = kwargs["value"]
             return
 
         if self.valid_values:
-            self.value = self.valid_values[0]
+            self._value = self.valid_values[0]
             return
 
-        self.value = DEFAULT_FOR_TYPE.get(self.format, None)
+        self._value = DEFAULT_FOR_TYPE.get(self.format, None)
 
         if self.minValue:
-            if not self.value:
-                self.value = self.minValue
-            self.value = max(self.value, self.minValue)
+            if not self._value:
+                self._value = self.minValue
+            self._value = max(self._value, self.minValue)
 
         if self.maxValue:
-            if not self.value:
-                self.value = self.maxValue
-            self.value = min(self.value, self.maxValue)
+            if not self._value:
+                self._value = self.maxValue
+            self._value = min(self._value, self.maxValue)
 
     def _get_configuration(
-        self, kwargs: Dict[str, Any], key: str, default: Optional[Any] = None,
+        self,
+        kwargs: Dict[str, Any],
+        key: str,
+        default: Optional[Any] = None,
     ) -> Optional[Any]:
         if key in kwargs:
             return kwargs[key]
@@ -132,13 +134,30 @@ class Characteristic(ToDictMixin):
 
     def set_value(self, new_val):
         """
-        This function sets the value of this characteristic. Permissions are checked first
-
-        :param new_val:
-        :raises CharacteristicPermissionError: if the characteristic cannot be written
+        This function sets the value of this characteristic.
         """
-        if CharacteristicPermissions.paired_write not in self.perms:
-            raise CharacteristicPermissionError(HapStatusCodes.CANT_WRITE_READ_ONLY)
+        self._value = new_val
+
+    @property
+    def value(self):
+        if self.format == CharacteristicFormats.tlv8:
+            extra_data = characteristics.get(self.type, {})
+
+            new_val = base64.b64decode(self._value)
+            struct = extra_data.get("struct")
+            if struct:
+                if extra_data.get("array"):
+                    return [struct.decode(new_val) for new_val in tlv_array(new_val)]
+                else:
+                    return struct.decode(new_val)
+
+        return self._value
+
+    @value.setter
+    def value(self, value):
+        self.set_value(value)
+
+    def validate_value(self, new_val):
         try:
             # convert input to python int if it is any kind of int
             if self.format in [
@@ -201,29 +220,7 @@ class Characteristic(ToDictMixin):
             if len(new_val) > self.maxLen:
                 raise FormatError(HapStatusCodes.INVALID_VALUE)
 
-        self.value = new_val
-
-    def set_value_from_ble(self, value):
-        if self.format == CharacteristicFormats.bool:
-            value = struct.unpack("?", value)[0]
-        elif self.format == CharacteristicFormats.uint8:
-            value = struct.unpack("B", value)[0]
-        elif self.format == CharacteristicFormats.uint16:
-            value = struct.unpack("H", value)[0]
-        elif self.format == CharacteristicFormats.uint32:
-            value = struct.unpack("I", value)[0]
-        elif self.format == CharacteristicFormats.uint64:
-            value = struct.unpack("Q", value)[0]
-        elif self.format == CharacteristicFormats.int:
-            value = struct.unpack("i", value)[0]
-        elif self.format == CharacteristicFormats.float:
-            value = struct.unpack("f", value)[0]
-        elif self.format == CharacteristicFormats.string:
-            value = value.decode("UTF-8")
-        else:
-            value = value.hex()
-
-        self.set_value(value)
+        return new_val
 
     def get_value(self):
         """
@@ -238,39 +235,6 @@ class Characteristic(ToDictMixin):
             raise CharacteristicPermissionError(HapStatusCodes.CANT_READ_WRITE_ONLY)
         return self.value
 
-    def get_value_for_ble(self):
-        value = self.get_value()
-
-        if self.format == CharacteristicFormats.bool:
-            try:
-                val = strtobool(str(value))
-            except ValueError:
-                raise FormatError(f'"{value}" is no valid "{self.format}"!')
-
-            value = struct.pack("?", val)
-        elif self.format == CharacteristicFormats.int:
-            value = struct.pack("i", int(value))
-        elif self.format == CharacteristicFormats.float:
-            value = struct.pack("f", float(value))
-        elif self.format == CharacteristicFormats.string:
-            value = value.encode()
-
-        return value
-
-    def get_meta(self):
-        """
-        This method returns a dict of meta information for this characteristic. This includes at least the format of
-        the characteristic but may contain any other specific attribute.
-
-        :return: a dict
-        """
-        tmp = {"format": self.format}
-        # TODO implement handling of already defined maxLen (upto 256!)
-        if self.format == CharacteristicFormats.string:
-            tmp["maxLen"] = 64
-        # TODO implement handling of other fields! eg maxDataLen
-        return tmp
-
     def to_accessory_and_service_list(self):
         d = {
             "type": self.type,
