@@ -16,7 +16,6 @@
 
 from typing import TYPE_CHECKING, Iterable, Optional
 
-from aiohomekit.model import ToDictMixin
 from aiohomekit.model.characteristics import Characteristic, CharacteristicsTypes
 from aiohomekit.model.characteristics.characteristic import check_convert_value
 from aiohomekit.model.services.data import services
@@ -51,7 +50,7 @@ class Characteristics:
         return next(self.filter(char_types=char_types))
 
 
-class Service(ToDictMixin):
+class Service:
     def __init__(
         self,
         accessory: "Accessory",
@@ -72,7 +71,7 @@ class Service(ToDictMixin):
 
         if name:
             char = self.add_char(CharacteristicsTypes.NAME)
-            char.value = name
+            char.set_value(name)
 
         if add_required:
             for required in services[self.type]["required"]:
